from visual import *
import string
import os
import os.path

print """
Choose an orbit situation.
During an orbit, click to stop and choose another.
Note that two of the situations differ only very
  slightly in the initial value of x.
The intent of the program is to demonstrate the
  great complexity of motion with just 3 objects,
  despite simple rules (Newtonian mechanics), and
  the high sensitivity to initial conditions in
  such a system.
"""

scene.width = sw = 600
scene.height = sh = 600
G = 1. # gravitational constant in arbitrary units
MTOR = 0.07 # convert mass to radius by MTOR*mass**(1./3.)

def orbit(filepath):
    stars = []
    stars.append(sphere(pos=(-0.5,0,0), vel=vector(0,0.6,0), mass=2.,
                        color=color.red, visible=0))
    stars.append(sphere(pos=(0.5,0,0), vel=vector(0,-1.2,0), mass=1.,
                        color=color.blue, visible=0))
    stars.append(sphere(pos=(-2,0,0,), vel=vector(0,1,0), mass=0.1,
                        color=color.green, visible=0))
    for s in stars:
        s.trail = curve(color=s.color)

    f = open(filepath, 'r')
    data = f.read()
    words = string.split(data)
    # File format: obsolete ID, followed by three sets of 8 numbers;
    # x, y, vx, vy, mass, radius (not used), color code (not used), 0
    for nn in range(3):
        base = 8*nn+1
        stars[nn].pos = vector(float(words[base+0]),float(words[base+1]),0)
        stars[nn].vel = vector(float(words[base+2]),float(words[base+3]),0)
        stars[nn].mass = float(words[base+4])

    ptotal = vector()
    mtotal = 0
    for s in stars:
        s.radius = MTOR*s.mass**(1./3.)
        s.p = s.mass*s.vel
        s.visible = 1
        ptotal = ptotal+s.p
        mtotal = mtotal+s.mass
    for s in stars:
        s.p = s.p-ptotal/mtotal

    scene.range = (sw/sh)*max(2., 1.2*stars[2].pos.mag)
    dt = 0.01

    while 1:
        rate(200)
        if scene.mouse.clicked:
            for s in stars:
                s.visible = 0
                s.trail.visible = 0
            return
        
        for s1 in stars:  # Find force acting on star "s1"
            F = vector(0,0,0) # We will add up all the forces on "s1" 
            for s2 in stars:
                if s2 == s1: continue # All stars but "s1"
                r12 = s2.pos-s1.pos
                if r12.mag > s1.radius+s2.radius:
                    # Add up all vector forces acting on star "s1"
                    F = F + (G*s1.mass*s2.mass/mag(r12)**2)*norm(r12)
                else:
                    while 1: # Stop plotting if there is a collision
                        if scene.mouse.clicked:
                            for s in stars:
                                s.visible = 0
                                s.trail.visible = 0
                            return
            s1.p = s1.p+F*dt  # Apply net force to star "s1"
            
        for s in stars: # After updating momenta, update positions
            s.pos = s.pos+(s.p/s.mass)*dt
            s.trail.append(pos=s.pos)

#########################################################################
# Set up menu of situation files to choose from

dir = '.'
situations = os.listdir(dir)
Nsituations = len(situations)
x0 = -0.2
y0 = +1.
scene.range = y0
dy = (2.*y0/Nsituations)
x = x0+0.15
y = y0-dy-0.03
buttons = []
for nn in range(Nsituations):
    if situations[nn][-3:] == '.py': continue
    buttons.append(box(pos=(x0+0.06,y,0), length=0.06, height=0.06,
                       width=0.01, color=color.blue))
    buttons[-1].title = label(pos=buttons[-1].pos+vector(0.03,0,0),
            text = situations[nn], xoffset=10, box=0, line=0)
    y = y-dy

while 1:
    filepath = ''
    while 1:
        if scene.mouse.clicked:
            m = scene.mouse.getclick()
            for nn in range(len(buttons)):
                if m.pick is buttons[nn]:
                    filepath = os.path.join(dir, buttons[nn].title.text)
                    break
        if filepath <> '': break
        
    for b in buttons:
        b.visible = 0
        b.title.visible = 0

    orbit(filepath)

    scene.range = y0*sw/sh
    for b in buttons:
        b.visible = 1
        b.title.visible = 1




